from __future__ import annotations

from typing import Generic, Optional, Tuple

from typing_extensions import Any, TypeVar

from . import constants, types, validators

T = TypeVar("T")


class BaseHttpClient(Generic[T]):
    __slots__ = ("session", "settings")

    def __init__(
        self, session: Optional[T], settings: validators.Settings
    ) -> None:
        self.session = session
        self.settings = settings

    def _get_params_and_headers(
        self, params: types.Params
    ) -> Tuple[types.Params, types.Headers]:
        if self.settings.lang:
            params = (
                {"lang": self.settings.lang, **params}
                if params
                else {"lang": self.settings.lang}
            )
        token = self.settings.token or constants.DEFAULT_TOKEN
        headers = {"X-Gismeteo-Token": token}
        return params, headers


THttpClient = TypeVar("THttpClient", bound=BaseHttpClient[Any])



from __future__ import annotations

from abc import ABCMeta, abstractmethod
from typing import Type, Union

from .. import models, types, validators


class StepNMixin(metaclass=ABCMeta):
    __slots__ = ()

    @property
    @abstractmethod
    def _endpoint(self) -> str:
        pass

    @property
    @abstractmethod
    def _model(self) -> types.StepNModel:
        pass

    @property
    @abstractmethod
    def _days_validator(
        self,
    ) -> Type[
        Union[validators.Step3Days, validators.Step6Days, validators.Step24Days]
    ]:
        pass


class Step3Mixin(StepNMixin):
    __slots__ = ()

    @property
    def _endpoint(self) -> str:
        return "weather/forecast"

    @property
    def _model(self) -> Type[models.step3.Model]:
        return models.step3.Model

    @property
    def _days_validator(self) -> Type[validators.Step3Days]:
        return validators.Step3Days


class Step6Mixin(StepNMixin):
    __slots__ = ()

    @property
    def _endpoint(self) -> str:
        return "weather/forecast/by_day_part"

    @property
    def _model(self) -> Type[models.step6.Model]:
        return models.step6.Model

    @property
    def _days_validator(self) -> Type[validators.Step6Days]:
        return validators.Step6Days


class Step24Mixin(StepNMixin):
    __slots__ = ()

    @property
    def _endpoint(self) -> str:
        return "weather/forecast/aggregate"

    @property
    def _model(self) -> Type[models.step24.Model]:
        return models.step24.Model

    @property
    def _days_validator(self) -> Type[validators.Step24Days]:
        return validators.Step24Days



from __future__ import annotations

from ipaddress import IPv4Address
from typing import Optional

try:
    from pydantic.v1 import BaseModel, Field
except ImportError:  # pragma: no cover
    from pydantic import BaseModel, Field  # type: ignore[assignment]

from . import types


class Settings(BaseModel):
    lang: Optional[types.Lang] = Field(...)
    token: Optional[str] = Field(...)

    class Config:
        anystr_strip_whitespace = True
        validate_assignment = True


class Coordinates(BaseModel):
    latitude: float = Field(ge=-90, le=90)
    longitude: float = Field(ge=-180, le=180)


class SearchLimit(BaseModel):
    __root__: int = Field(ge=1, le=36)


class IPAddress(BaseModel):
    __root__: IPv4Address


class Query(BaseModel):
    __root__: str


class LocalityID(BaseModel):
    __root__: int = Field(ge=1)


class Step3Days(BaseModel):
    __root__: int = Field(ge=1, le=10)


class Step6or24Days(BaseModel):
    __root__: int = Field(ge=3, le=10)


Step6Days = Step6or24Days
Step24Days = Step6or24Days



from __future__ import annotations

from typing import Mapping, Optional, Type, Union

from typing_extensions import Literal, TypeAlias, TypeVar

from . import models

Lang: TypeAlias = Literal["ru", "en", "ua", "lt", "lv", "pl", "ro"]
"""Языки, поддерживаемые Gismeteo API."""
Step3Days: TypeAlias = Literal[1, 2, 3, 4, 5, 6, 7, 8, 9, 10]
"""Допустимые значения аргумента days у погоды с шагом 3 часа."""
Step6or24Days: TypeAlias = Literal[3, 4, 5, 6, 7, 8, 9, 10]
Step6Days: TypeAlias = Step6or24Days
"""Допустимые значения аргумента days у погоды с шагом 6 часов."""
Step24Days: TypeAlias = Step6or24Days
"""Допустимые значения аргумента days у погоды с шагом 24 часа."""
StepNDays: TypeAlias = Union[Step3Days, Step6Days, Step24Days]
Params: TypeAlias = Optional[Mapping[str, str]]
Headers: TypeAlias = Optional[Mapping[str, str]]
SearchLimit: TypeAlias = Literal[
    1,
    2,
    3,
    4,
    5,
    6,
    7,
    8,
    9,
    10,
    11,
    12,
    13,
    14,
    15,
    16,
    17,
    18,
    19,
    20,
    21,
    22,
    23,
    24,
    25,
    26,
    27,
    28,
    29,
    30,
    31,
    32,
    33,
    34,
    35,
    36,
]
"""Допустимые значения аргумента limit у поиска по координатам."""

StepNModel: TypeAlias = Type[
    Union[models.step3.Model, models.step6.Model, models.step24.Model]
]
TDays = TypeVar("TDays", Step3Days, Step6Days, Step24Days)
TStepNModel = TypeVar(
    "TStepNModel", models.step3.Model, models.step6.Model, models.step24.Model
)
TStepNModelItem = TypeVar(
    "TStepNModelItem",
    models.step3.ModelItem,
    models.step6.ModelItem,
    models.step24.ModelItem,
)



from __future__ import annotations

from abc import ABCMeta, abstractmethod
from typing import Generic

from . import http


class EndpointABC(Generic[http.THttpClient], metaclass=ABCMeta):
    __slots__ = ("_session",)

    def __init__(self, session: http.THttpClient) -> None:
        self._session = session

    @property
    @abstractmethod
    def _endpoint(self) -> str:
        pass



from __future__ import annotations

from typing import List, Optional

try:
    from pydantic.v1 import BaseModel, Field
except ImportError:  # pragma: no cover
    from pydantic import BaseModel, Field  # type: ignore[assignment]


class District(BaseModel):
    name: str
    name_p: Optional[str] = Field(default=None, alias="nameP")


class Country(BaseModel):
    name: str
    code: str
    name_p: Optional[str] = Field(default=None, alias="nameP")


class SubDistrict(BaseModel):
    name: str
    name_p: Optional[str] = Field(default=None, alias="nameP")


class ModelItem(BaseModel):
    district: Optional[District] = None
    id: int
    sub_district: Optional[SubDistrict] = None
    url: str
    name_p: Optional[str] = Field(default=None, alias="nameP")
    name: str
    rate: int
    weight: int
    kind: str
    country: Country


class Model(BaseModel):
    __root__: List[ModelItem]



from __future__ import annotations

from abc import abstractmethod
from typing import Tuple, Type, Union

from .. import http, types, validators
from ..endpoint import EndpointABC


class StepNABC(EndpointABC[http.THttpClient]):
    __slots__ = ()

    @property
    @abstractmethod
    def _model(self) -> types.StepNModel:
        pass

    @property
    @abstractmethod
    def _days_validator(
        self,
    ) -> Type[
        Union[validators.Step3Days, validators.Step6Days, validators.Step24Days]
    ]:
        pass

    def _get_params_by_coordinates(
        self, latitude: float, longitude: float, *, days: types.StepNDays
    ) -> Tuple[str, types.Params]:
        coords_validator = validators.Coordinates(
            latitude=latitude, longitude=longitude
        )
        days_validator = self._days_validator.parse_obj(days)
        params = {
            "latitude": str(coords_validator.latitude),
            "longitude": str(coords_validator.longitude),
            "days": str(days_validator.__root__),
        }
        return self._endpoint, params

    def _get_params_by_id(
        self, id: int, *, days: types.StepNDays  # noqa: A002
    ) -> Tuple[str, types.Params]:
        id_validator = validators.LocalityID.parse_obj(id)
        url = f"{self._endpoint}/{id_validator.__root__}"
        days_validator = self._days_validator.parse_obj(days)
        params = {"days": str(days_validator.__root__)}
        return url, params



from __future__ import annotations

from ipaddress import IPv4Address
from typing import Union

from . import http, types, validators
from .endpoint import EndpointABC


class SearchBase(EndpointABC[http.THttpClient]):
    __slots__ = ()

    @property
    def _endpoint(self) -> str:
        return "search/cities"

    @staticmethod
    def _get_params_by_coordinates(
        latitude: float, longitude: float, *, limit: types.SearchLimit
    ) -> types.Params:
        coords_validator = validators.Coordinates(
            latitude=latitude, longitude=longitude
        )
        limit_validator = validators.SearchLimit.parse_obj(limit)
        return {
            "latitude": str(coords_validator.latitude),
            "longitude": str(coords_validator.longitude),
            "limit": str(limit_validator.__root__),
        }

    @staticmethod
    def _get_params_by_ip(ip: Union[IPv4Address, str]) -> types.Params:
        ip_validator = validators.IPAddress.parse_obj(ip)
        return {"ip": str(ip_validator.__root__)}

    @staticmethod
    def _get_params_by_query(query: str) -> types.Params:
        query_validator = validators.Query.parse_obj(query)
        return {"query": query_validator.__root__}



from __future__ import annotations

from typing import Tuple

from . import http, types, validators
from .endpoint import EndpointABC


class CurrentBase(EndpointABC[http.THttpClient]):
    __slots__ = ()

    @property
    def _endpoint(self) -> str:
        return "weather/current"

    def _get_params_by_coordinates(
        self, latitude: float, longitude: float
    ) -> Tuple[str, types.Params]:
        coords_validator = validators.Coordinates(
            latitude=latitude, longitude=longitude
        )
        params = {
            "latitude": str(coords_validator.latitude),
            "longitude": str(coords_validator.longitude),
        }
        return self._endpoint, params

    def _get_params_by_id(
        self, id: int  # noqa: A002
    ) -> Tuple[str, types.Params]:
        id_validator = validators.LocalityID.parse_obj(id)
        url = f"{self._endpoint}/{id_validator.__root__}"
        return url, None



"""Pydantic модели ответов Gismeteo API."""
from __future__ import annotations

from . import (
    current,
    search_by_coordinates,
    search_by_ip,
    search_by_query,
    step3,
    step6,
    step24,
)

__all__ = (
    "current",
    "search_by_coordinates",
    "search_by_ip",
    "search_by_query",
    "step3",
    "step6",
    "step24",
)
